@@ -440,21 +440,12 @@ set_item(bitarrayobject *self, Py_ssize_t i, PyObject *v)
     int vi;
 
     assert(0 <= i && i < self->nbits);
-    vi = PyObject_IsTrue(v);
-    if (vi < 0)
+    if ((vi = pybit_as_int(v)) < 0)
         return -1;
     setbit(self, i, vi);
     return 0;
 }
 
-static int
-append_item(bitarrayobject *self, PyObject *item)
-{
-    if (resize(self, self->nbits + 1) < 0)
-        return -1;
-    return set_item(self, self->nbits - 1, item);
-}
-
 static int
 extend_bitarray(bitarrayobject *self, bitarrayobject *other)
 {
@@ -476,30 +467,36 @@ extend_bitarray(bitarrayobject *self, bitarrayobject *other)
 static int
 extend_iter(bitarrayobject *self, PyObject *iter)
 {
+    const Py_ssize_t original_nbits = self->nbits;
     PyObject *item;
 
     assert(PyIter_Check(iter));
     while ((item = PyIter_Next(iter))) {
-        if (append_item(self, item) < 0) {
-            Py_DECREF(item);
-            return -1;
-        }
+        if (resize(self, self->nbits + 1) < 0)
+            goto error;
+        if (set_item(self, self->nbits - 1, item) < 0)
+            goto error;
         Py_DECREF(item);
     }
     if (PyErr_Occurred())
         return -1;
 
     return 0;
+ error:
+    Py_DECREF(item);
+    resize(self, original_nbits);
+    return -1;
 }
 
 static int
-extend_list(bitarrayobject *self, PyObject *list)
+extend_sequence(bitarrayobject *self, PyObject *sequence)
 {
+    const Py_ssize_t original_nbits = self->nbits;
     PyObject *item;
     Py_ssize_t n, i;
 
-    assert(PyList_Check(list));
-    n = PyList_GET_SIZE(list);
+    assert(PySequence_Check(sequence));
+    n = PySequence_Size(sequence);
     if (n == 0)
         return 0;
 
@@ -507,11 +504,13 @@ extend_list(bitarrayobject *self, PyObject *list)
         return -1;
 
     for (i = 0; i < n; i++) {
-        item = PyList_GET_ITEM(list, i);
-        if (item == NULL)
-            return -1;
-        if (set_item(self, self->nbits - n + i, item) < 0)
+        item = PySequence_GetItem(sequence, i);
+        if (item == NULL || set_item(self, self->nbits - n + i, item) < 0) {
+            Py_XDECREF(item);
+            resize(self, original_nbits);
             return -1;
+        }
+        Py_DECREF(item);
     }
     return 0;
 }
@@ -576,9 +575,6 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
     if (bitarray_Check(obj))                              /* bitarray */
         return extend_bitarray(self, (bitarrayobject *) obj);
 
-    if (PyList_Check(obj))                                    /* list */
-        return extend_list(self, obj);
-
     if (PyBytes_Check(obj)) {                             /* bytes 01 */
 #ifdef IS_PY3K
         PyErr_SetString(PyExc_TypeError,
@@ -593,6 +589,9 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
     if (PyUnicode_Check(obj))                           /* unicode 01 */
         return extend_unicode01(self, obj);
 
+    if (PySequence_Check(obj))                            /* sequence */
+        return extend_sequence(self, obj);
+
     if (PyIter_Check(obj))                                    /* iter */
         return extend_iter(self, obj);
 
@@ -652,21 +651,12 @@ normalize_index(Py_ssize_t n, Py_ssize_t *i)
 static PyObject *
 bitarray_length(bitarrayobject *self)
 {
-    if (PyErr_WarnEx(PyExc_DeprecationWarning,
-                     "self.length() has been deprecated since 1.5.1, "
-                     "use len(self) instead", 1) < 0)
-        return NULL;
-
-    return PyLong_FromSsize_t(self->nbits);
+    PyErr_SetString(PyExc_NotImplementedError,
+                    "self.length() has been deprecated since 1.5.1, "
+                    "and was removed in 2.0.0.  Use len(self) instead.");
+    return NULL;
 }
 
-PyDoc_STRVAR(length_doc,
-"length() -> int\n\
-\n\
-Return the length - a.length() is the same as len(a).\n\
-Deprecated since 1.5.1, use len().");
-
-
 static PyObject *
 bitarray_copy(bitarrayobject *self)
 {
@@ -690,15 +680,14 @@ Return a copy of the bitarray.");
 static PyObject *
 bitarray_count(bitarrayobject *self, PyObject *args)
 {
-    PyObject *x = Py_True;
+    PyObject *v = Py_True;
     Py_ssize_t start = 0, stop = self->nbits;
     int vi;
 
-    if (!PyArg_ParseTuple(args, "|Onn:count", &x, &start, &stop))
+    if (!PyArg_ParseTuple(args, "|Onn:count", &v, &start, &stop))
         return NULL;
 
-    vi = PyObject_IsTrue(x);
-    if (vi < 0)
+    if ((vi = pybit_as_int(v)) < 0)
         return NULL;
 
     normalize_index(self->nbits, &start);
@@ -708,30 +697,28 @@ bitarray_count(bitarrayobject *self, PyObject *args)
 }
 
 PyDoc_STRVAR(count_doc,
-"count(value=True, start=0, stop=<end of array>, /) -> int\n\
+"count(value=1, start=0, stop=<end of array>, /) -> int\n\
 \n\
-Count the number of occurrences of bool(value) in the bitarray.");
+Count the number of occurrences of `value` in the bitarray.");
 
 
 static PyObject *
 bitarray_index(bitarrayobject *self, PyObject *args)
 {
-    PyObject *x;
     Py_ssize_t start = 0, stop = self->nbits, i;
+    PyObject *value;
     int vi;
 
-    if (!PyArg_ParseTuple(args, "O|nn:index", &x, &start, &stop))
+    if (!PyArg_ParseTuple(args, "O|nn:index", &value, &start, &stop))
         return NULL;
 
-    vi = PyObject_IsTrue(x);
-    if (vi < 0)
+    if ((vi = pybit_as_int(value)) < 0)
         return NULL;
 
     normalize_index(self->nbits, &start);
     normalize_index(self->nbits, &stop);
 
-    i = findfirst(self, vi, start, stop);
-    if (i < 0)
+    if ((i = findfirst(self, vi, start, stop)) < 0)
         return PyErr_Format(PyExc_ValueError, "%d is not in bitarray", vi);
 
     return PyLong_FromSsize_t(i);
@@ -740,7 +727,7 @@ bitarray_index(bitarrayobject *self, PyObject *args)
 PyDoc_STRVAR(index_doc,
 "index(value, start=0, stop=<end of array>, /) -> int\n\
 \n\
-Return index of the first occurrence of `bool(value)` in the bitarray.\n\
+Return index of the first occurrence of `value` in the bitarray.\n\
 Raises `ValueError` if the value is not present.");
 
 
@@ -755,9 +742,9 @@ bitarray_extend(bitarrayobject *self, PyObject *obj)
 PyDoc_STRVAR(extend_doc,
 "extend(iterable, /)\n\
 \n\
-Extend bitarray by appending the truth value of each element given\n\
-by iterable.  If a string is provided, each `0` and `1` are appended\n\
-as bits (whitespace is ignored).");
+Append all the items from `iterable` to the end of the bitarray.\n\
+If the iterable is a string, each `0` and `1` are appended as\n\
+bits (ignoring whitespace).");
 
 
 static PyObject *
@@ -848,17 +835,22 @@ Return the bit endianness of the bitarray as a string (`little` or `big`).");
 
 
 static PyObject *
-bitarray_append(bitarrayobject *self, PyObject *v)
+bitarray_append(bitarrayobject *self, PyObject *value)
 {
-    if (append_item(self, v) < 0)
+    int vi;
+
+    if ((vi = pybit_as_int(value)) < 0)
         return NULL;
+    if (resize(self, self->nbits + 1) < 0)
+        return NULL;
+    setbit(self, self->nbits - 1, vi);
     Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(append_doc,
 "append(item, /)\n\
 \n\
-Append the truth value `bool(item)` to the end of the bitarray.");
+Append `item` to the end of the bitarray.");
 
 
 static PyObject *
@@ -1050,7 +1042,7 @@ PyDoc_STRVAR(bytereverse_doc,
 \n\
 For all bytes representing the bitarray, reverse the bit order (in-place).\n\
 Note: This method changes the actual machine values representing the\n\
-bitarray; it does not change the endianness of the bitarray object.");
+bitarray; it does *not* change the endianness of the bitarray object.");
 
 
 static PyObject *
@@ -1058,10 +1050,8 @@ bitarray_setall(bitarrayobject *self, PyObject *v)
 {
     int vi;
 
-    vi = PyObject_IsTrue(v);
-    if (vi < 0)
+    if ((vi = pybit_as_int(v)) < 0)
         return NULL;
-
     memset(self->ob_item, vi ? 0xff : 0x00, (size_t) Py_SIZE(self));
     Py_RETURN_NONE;
 }
@@ -1069,7 +1059,8 @@ bitarray_setall(bitarrayobject *self, PyObject *v)
 PyDoc_STRVAR(setall_doc,
 "setall(value, /)\n\
 \n\
-Set all bits in the bitarray to `bool(value)`.");
+Set all elements in the bitarray to `value`.\n\
+Note that `a.setall(value)` is equivalent to `a[:] = value`.");
 
 
 static PyObject *
@@ -1104,22 +1095,17 @@ Sort the bits in the array (in-place).");
 
 
 static PyObject *
-bitarray_tolist(bitarrayobject *self, PyObject *args)
+bitarray_tolist(bitarrayobject *self)
 {
     PyObject *list, *item;
     Py_ssize_t i;
-    int as_ints = 0;
-
-    if (!PyArg_ParseTuple(args, "|i:tolist", &as_ints))
-        return NULL;
 
     list = PyList_New(self->nbits);
     if (list == NULL)
         return NULL;
 
     for (i = 0; i < self->nbits; i++) {
-        item = as_ints ? PyLong_FromLong(GETBIT(self, i)) :
-                         PyBool_FromLong(GETBIT(self, i));
+        item = PyLong_FromLong(GETBIT(self, i));
         if (item == NULL)
             return NULL;
         if (PyList_SetItem(list, i, item) < 0)
@@ -1129,10 +1115,9 @@ bitarray_tolist(bitarrayobject *self, PyObject *args)
 }
 
 PyDoc_STRVAR(tolist_doc,
-"tolist(as_ints=False, /) -> list\n\
+"tolist() -> list\n\
 \n\
-Return a list with the items (False or True) in the bitarray.\n\
-The optional parameter, changes the items in the list to integers (0 or 1).\n\
+Return a list with the items (0 or 1) in the bitarray.\n\
 Note that the list object being created will require 32 or 64 times more\n\
 memory (depending on the machine architecture) than the bitarray object,\n\
 which may cause a memory error if the bitarray is very large.");
@@ -1244,7 +1229,7 @@ PyDoc_STRVAR(fromfile_doc,
 Extend bitarray with up to n bytes read from the file object f.\n\
 When n is omitted or negative, reads all data until EOF.\n\
 When n is provided and positive but exceeds the data available,\n\
-EOFError is raised (but the available data is still read and appended.");
+`EOFError` is raised (but the available data is still read and appended.");
 
 
 static PyObject *
@@ -1293,7 +1278,7 @@ static PyObject *
 bitarray_unpack(bitarrayobject *self, PyObject *args, PyObject *kwds)
 {
     static char *kwlist[] = {"zero", "one", NULL};
-    char zero = 0x00, one = 0xff;
+    char zero = 0x00, one = 0x01;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "|cc:unpack", kwlist,
                                      &zero, &one))
@@ -1303,7 +1288,7 @@ bitarray_unpack(bitarrayobject *self, PyObject *args, PyObject *kwds)
 }
 
 PyDoc_STRVAR(unpack_doc,
-"unpack(zero=b'\\x00', one=b'\\xff') -> bytes\n\
+"unpack(zero=b'\\x00', one=b'\\x01') -> bytes\n\
 \n\
 Return bytes containing one character for each bit in the bitarray,\n\
 using the specified mapping.");
@@ -1362,8 +1347,7 @@ bitarray_repr(bitarrayobject *self)
         return NULL;
     }
 
-    str = (char *) PyMem_Malloc(strsize);
-    if (str == NULL)
+    if ((str = (char *) PyMem_Malloc(strsize)) == NULL)
         return PyErr_NoMemory();
 
     /* add "bitarray('......')" to str */
@@ -1386,23 +1370,25 @@ bitarray_insert(bitarrayobject *self, PyObject *args)
 {
     Py_ssize_t i;
     PyObject *v;
+    int vi;
 
     if (!PyArg_ParseTuple(args, "nO:insert", &i, &v))
         return NULL;
 
     normalize_index(self->nbits, &i);
 
+    if ((vi = pybit_as_int(v)) < 0)
+        return NULL;
     if (insert_n(self, i, 1) < 0)
         return NULL;
-    if (set_item(self, i, v) < 0)
-        return NULL;
+    setbit(self, i, vi);
     Py_RETURN_NONE;
 }
 
 PyDoc_STRVAR(insert_doc,
 "insert(index, value, /)\n\
 \n\
-Insert `bool(value)` into the bitarray before index.");
+Insert `value` into the bitarray before `index`.");
 
 
 static PyObject *
@@ -1429,7 +1415,7 @@ bitarray_pop(bitarrayobject *self, PyObject *args)
     vi = GETBIT(self, i);
     if (delete_n(self, i, 1) < 0)
         return NULL;
-    return PyBool_FromLong(vi);
+    return PyLong_FromLong(vi);
 }
 
 PyDoc_STRVAR(pop_doc,
@@ -1440,17 +1426,15 @@ Raises `IndexError` if bitarray is empty or index is out of range.");
 
 
 static PyObject *
-bitarray_remove(bitarrayobject *self, PyObject *v)
+bitarray_remove(bitarrayobject *self, PyObject *value)
 {
     Py_ssize_t i;
     int vi;
 
-    vi = PyObject_IsTrue(v);
-    if (vi < 0)
+    if ((vi = pybit_as_int(value)) < 0)
         return NULL;
 
-    i = findfirst(self, vi, 0, self->nbits);
-    if (i < 0)
+    if ((i = findfirst(self, vi, 0, self->nbits)) < 0)
         return PyErr_Format(PyExc_ValueError, "%d not in bitarray", vi);
 
     if (delete_n(self, i, 1) < 0)
@@ -1461,7 +1445,7 @@ bitarray_remove(bitarrayobject *self, PyObject *v)
 PyDoc_STRVAR(remove_doc,
 "remove(value, /)\n\
 \n\
-Remove the first occurrence of `bool(value)` in the bitarray.\n\
+Remove the first occurrence of `value` in the bitarray.\n\
 Raises `ValueError` if item is not present.");
 
 
@@ -1505,8 +1489,7 @@ bitarray_concat(bitarrayobject *self, PyObject *other)
 {
     PyObject *res;
 
-    res = bitarray_copy(self);
-    if (res == NULL)
+    if ((res = bitarray_copy(self)) == NULL)
         return NULL;
 
     if (extend_dispatch((bitarrayobject *) res, other) < 0) {
@@ -1521,8 +1504,7 @@ bitarray_repeat(bitarrayobject *self, Py_ssize_t n)
 {
     PyObject *res;
 
-    res = bitarray_copy(self);
-    if (res == NULL)
+    if ((res = bitarray_copy(self)) == NULL)
         return NULL;
 
     if (repeat((bitarrayobject *) res, n) < 0) {
@@ -1539,7 +1521,7 @@ bitarray_item(bitarrayobject *self, Py_ssize_t i)
         PyErr_SetString(PyExc_IndexError, "bitarray index out of range");
         return NULL;
     }
-    return PyBool_FromLong(GETBIT(self, i));
+    return PyLong_FromLong(GETBIT(self, i));
 }
 
 static int
@@ -1556,22 +1538,22 @@ bitarray_ass_item(bitarrayobject *self, Py_ssize_t i, PyObject *value)
         return set_item(self, i, value);
 }
 
-/* return 1 if 'item' (which can be an int or bitarray) is in self,
+/* return 1 if value (which can be an int or bitarray) is in self,
    0 otherwise, and -1 on error */
 static int
-bitarray_contains(bitarrayobject *self, PyObject *item)
+bitarray_contains(bitarrayobject *self, PyObject *value)
 {
-    if (PyIndex_Check(item)) {
+    if (PyIndex_Check(value)) {
         int vi;
 
-        vi = IntOrBool_AsInt(item);
+        vi = pybit_as_int(value);
         if (vi < 0)
             return -1;
         return findfirst(self, vi, 0, self->nbits) >= 0;
     }
 
-    if (bitarray_Check(item))
-        return search(self, (bitarrayobject *) item, 0) >= 0;
+    if (bitarray_Check(value))
+        return search(self, (bitarrayobject *) value, 0) >= 0;
 
     PyErr_SetString(PyExc_TypeError, "bitarray or bool expected");
     return -1;
@@ -1672,8 +1654,7 @@ setslice_bitarray(bitarrayobject *self, PyObject *slice, PyObject *array)
     increase = aa->nbits - slicelength;
 
     if (aa == self) {  /* covers cases like a[2::] = a and a[::-1] = a */
-        array = bitarray_copy(aa);
-        if (array == NULL)
+        if ((array = bitarray_copy(aa)) == NULL)
             return -1;
         copy_array = 1;
     }
@@ -1711,20 +1692,19 @@ setslice_bitarray(bitarrayobject *self, PyObject *slice, PyObject *array)
     return res;
 }
 
-/* set the elements in self, specified by slice, to bool */
+/* set the elements in self, specified by slice, to value */
 static int
-setslice_bool(bitarrayobject *self, PyObject *slice, PyObject *bool)
+setslice_bool(bitarrayobject *self, PyObject *slice, PyObject *value)
 {
     Py_ssize_t start, stop, step, slicelength, i, j;
     int vi;
 
-    assert(PySlice_Check(slice) && PyIndex_Check(bool));
+    assert(PySlice_Check(slice) && PyIndex_Check(value));
     if (PySlice_GetIndicesEx(slice, self->nbits,
                              &start, &stop, &step, &slicelength) < 0)
         return -1;
 
-    vi = IntOrBool_AsInt(bool);
-    if (vi < 0)
+    if ((vi = pybit_as_int(value)) < 0)
         return -1;
 
     if (step == 1) {
@@ -1824,8 +1804,7 @@ bitarray_cpinvert(bitarrayobject *self)
 {
     PyObject *result;
 
-    result = bitarray_copy(self);
-    if (result == NULL)
+    if ((result = bitarray_copy(self)) == NULL)
         return NULL;
 
     invert((bitarrayobject *) result);
@@ -2366,8 +2345,7 @@ decodetree_todict(decodetreeobject *self)
 {
     PyObject *dict, *prefix;
 
-    dict = PyDict_New();
-    if (dict == NULL)
+    if ((dict = PyDict_New()) == NULL)
         return NULL;
 
     prefix = newbitarrayobject(&Bitarray_Type, 0, default_endian);
@@ -2491,13 +2469,11 @@ bitarray_decode(bitarrayobject *self, PyObject *obj)
         if (check_codedict(obj) < 0)
             return NULL;
 
-        tree = binode_make_tree(obj);
-        if (tree == NULL)
+        if ((tree = binode_make_tree(obj)) == NULL)
             return NULL;
     }
 
-    list = PyList_New(0);
-    if (list == NULL)
+    if ((list = PyList_New(0)) == NULL)
         goto error;
 
     while ((symbol = binode_traverse(tree, self, &index))) {
@@ -2550,8 +2526,7 @@ bitarray_iterdecode(bitarrayobject *self, PyObject *obj)
         if (check_codedict(obj) < 0)
             return NULL;
 
-        tree = binode_make_tree(obj);
-        if (tree == NULL)
+        if ((tree = binode_make_tree(obj)) == NULL)
             return NULL;
     }
 
@@ -2801,8 +2776,7 @@ static PyMethodDef bitarray_methods[] = {
      insert_doc},
     {"invert",       (PyCFunction) bitarray_invert,      METH_VARARGS,
      invert_doc},
-    {"length",       (PyCFunction) bitarray_length,      METH_NOARGS,
-     length_doc},
+    {"length",       (PyCFunction) bitarray_length,      METH_NOARGS, 0},
     {"pack",         (PyCFunction) bitarray_pack,        METH_O,
      pack_doc},
     {"pop",          (PyCFunction) bitarray_pop,         METH_VARARGS,
@@ -2822,7 +2796,7 @@ static PyMethodDef bitarray_methods[] = {
      sort_doc},
     {"tofile",       (PyCFunction) bitarray_tofile,      METH_O,
      tofile_doc},
-    {"tolist",       (PyCFunction) bitarray_tolist,      METH_VARARGS,
+    {"tolist",       (PyCFunction) bitarray_tolist,      METH_NOARGS,
      tolist_doc},
     {"tobytes",      (PyCFunction) bitarray_tobytes,     METH_NOARGS,
      tobytes_doc},
@@ -3043,7 +3017,7 @@ bitarrayiter_next(bitarrayiterobject *it)
     if (it->index < it->bao->nbits) {
         vi = GETBIT(it->bao, it->index);
         it->index++;
-        return PyBool_FromLong(vi);
+        return PyLong_FromLong(vi);
     }
     return NULL;  /* stop iteration */
 }
@@ -3199,8 +3173,7 @@ uninitialized.\n\
 \n\
 `str`: Create bitarray from a string of `0` and `1`.\n\
 \n\
-`iterable`: Create bitarray from a sequence, each element in the sequence\n\
-is converted to a bit using its truth value.\n\
+`iterable`: Create bitarray from iterable or sequence or integers 0 or 1.\n\
 \n\
 The optional keyword arguments `endian` specifies the bit endianness of the\n\
 created bitarray object.\n\
