@@ -170,17 +170,16 @@ find_last(bitarrayobject *a, int vi)
 static PyObject *
 r_index(PyObject *module, PyObject *args)
 {
-    PyObject *x = Py_True, *a;
+    PyObject *v = Py_True, *a;
     Py_ssize_t i;
     int vi;
 
-    if (!PyArg_ParseTuple(args, "O|O:rindex", &a, &x))
+    if (!PyArg_ParseTuple(args, "O|O:rindex", &a, &v))
         return NULL;
     if (ensure_bitarray(a) < 0)
         return NULL;
 
-    vi = PyObject_IsTrue(x);
-    if (vi < 0)
+    if ((vi = pybit_as_int(v)) < 0)
         return NULL;
 
     i = find_last((bitarrayobject *) a, vi);
@@ -191,9 +190,9 @@ r_index(PyObject *module, PyObject *args)
 }
 
 PyDoc_STRVAR(rindex_doc,
-"rindex(bitarray, value=True, /) -> int\n\
+"rindex(bitarray, value=1, /) -> int\n\
 \n\
-Return the rightmost index of `bool(value)` in bitarray.\n\
+Return the rightmost index of `value` in bitarray.\n\
 Raises `ValueError` if the value is not present.");
 
 /* --------------------------- unary functions ------------------------- */
@@ -214,14 +213,14 @@ parity(PyObject *module, PyObject *a)
         par ^= aa->ob_item[i];
 #undef aa
 
-    return PyBool_FromLong((long) bitcount_lookup[par] % 2);
+    return PyLong_FromLong((long) bitcount_lookup[par] % 2);
 }
 
 PyDoc_STRVAR(parity_doc,
-"parity(a, /) -> bool\n\
+"parity(a, /) -> int\n\
 \n\
-Return the parity of bitarray `a`.  This is equivalent\n\
-to `bool(a.count() % 2)` (but more efficient).");
+Return the parity of bitarray `a`.\n\
+This is equivalent to `a.count() % 2` (but more efficient).");
 
 /* --------------------------- binary functions ------------------------ */
 
@@ -320,7 +319,7 @@ subset(PyObject *module, PyObject *args)
 PyDoc_STRVAR(subset_doc,
 "subset(a, b, /) -> bool\n\
 \n\
-Return True if bitarray `a` is a subset of bitarray `b` (False otherwise).\n\
+Return `True` if bitarray `a` is a subset of bitarray `b`.\n\
 `subset(a, b)` is equivalent to `(a & b).count() == a.count()` but is more\n\
 efficient since we can stop as soon as one mismatch is found, and no\n\
 intermediate bitarray object gets created.");
@@ -479,10 +478,9 @@ static int
 digit_to_int(char c, int n)
 {
     if (n <= 16) {              /* base 2, 4, 8, 16 */
-        int i;
-
-        i  = hex_to_int(c);
-        return (0 <= i && i < n) ? i : -1;
+        int i = hex_to_int(c);
+        if (0 <= i && i < n)
+            return i;
     }
     if (n == 32) {              /* base 32 */
         if ('A' <= c && c <= 'Z')
@@ -505,6 +503,7 @@ digit_to_int(char c, int n)
     return -1;
 }
 
+/* return m = log2(n) for m = 1..6 */
 static int
 base_to_length(int n)
 {
@@ -534,12 +533,11 @@ ba2base(PyObject *module, PyObject *args)
     if ((m = base_to_length(n)) < 0)
         return NULL;
 
-    if (n == 32)
-        alphabet = base32_alphabet;
-    else if (n == 64)
-        alphabet = base64_alphabet;
-    else
-        alphabet = hexdigits;
+    switch (n) {
+    case 32: alphabet = base32_alphabet; break;
+    case 64: alphabet = base64_alphabet; break;
+    default: alphabet = hexdigits;
+    }
 
 #define aa  ((bitarrayobject *) a)
     if (aa->nbits % m)
@@ -547,8 +545,7 @@ ba2base(PyObject *module, PyObject *args)
                             "bitarray length not multiple of %d", m);
 
     strsize = aa->nbits / m;
-    str = (char *) PyMem_Malloc(strsize);
-    if (str == NULL)
+    if ((str = (char *) PyMem_Malloc(strsize)) == NULL)
         return PyErr_NoMemory();
 
     le = IS_LE(aa);
@@ -578,7 +575,7 @@ standard base 64 alphabet is used.");
 
 /* Translate ASCII digits into the bitarray's buffer.
    The (Python) arguments to this functions are:
-   - the base n, either 2, 4, 8, 16, 32 or 64    (n=2^m - m bits per digits)
+   - base n, one of 2, 4, 8, 16, 32, 64  (n=2^m   where m bits per digit)
    - bitarray (of length m * len(s)) whose buffer is written into
    - byte object s containing the ASCII digits
 */
