@@ -4,7 +4,7 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "1.9.1"
+#define BITARRAY_VERSION  "2.0.0"
 
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K
@@ -108,3 +108,21 @@ static const unsigned char bitcount_lookup[256] = {
     2,3,3,4,3,4,4,5,3,4,4,5,4,5,5,6,3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,
     3,4,4,5,4,5,5,6,4,5,5,6,5,6,6,7,4,5,5,6,5,6,6,7,5,6,6,7,6,7,7,8,
 };
+
+/* Interpret a PyObject (usually PyLong or PyBool) as a bit, return 0 or 1.
+   On error, return -1 and set error message. */
+static inline int
+pybit_as_int(PyObject *v)
+{
+    Py_ssize_t x;
+
+    x = PyNumber_AsSsize_t(v, NULL);
+    if (x == -1 && PyErr_Occurred())
+        return -1;
+
+    if (x < 0 || x > 1) {
+        PyErr_Format(PyExc_ValueError, "bit must be 0 or 1, got %zd", x);
+        return -1;
+    }
+    return (int) x;
+}
