@@ -5,6 +5,8 @@ from __future__ import absolute_import
 
 import os
 import sys
+import base64
+import binascii
 import shutil
 import tempfile
 import unittest
@@ -17,9 +19,10 @@ from bitarray import (bitarray, frozenbitarray, bits2bytes, decodetree,
 from bitarray.test_bitarray import Util
 
 from bitarray.util import (
-    zeros, urandom, pprint, make_endian, rindex, strip,
-    count_n, count_and, count_or, count_xor, subset, parity,
-    ba2hex, hex2ba, ba2int, int2ba, serialize, deserialize, huffman_code
+    zeros, urandom, pprint, make_endian, rindex, strip, count_n,
+    parity, count_and, count_or, count_xor, subset,
+    serialize, deserialize, ba2hex, hex2ba, ba2base, base2ba,
+    ba2int, int2ba, huffman_code
 )
 
 if sys.version_info[0] == 3:
@@ -45,6 +48,10 @@ class TestsZeros(unittest.TestCase):
 
             for n in range(100):
                 a = zeros(n)
+                self.assertEqual(len(a), n)
+                self.assertFalse(a.any())
+                self.assertEqual(a.count(1), 0)
+                self.assertEqual(a, n * bitarray('0'))
                 self.assertEqual(a, bitarray(n * '0'))
 
             for endian in 'big', 'little':
@@ -131,6 +138,20 @@ class TestsPPrint(unittest.TestCase):
         self.assertEqual(self.get_code_string(a), "frozenbitarray('01')\n")
         self.round_trip(a)
 
+    def test_formatting(self):
+        a = bitarray(200)
+        for width in range(40, 130, 10):
+            for n in range(1, 10):
+                f = StringIO()
+                pprint(a, stream=f, group=n, width=width)
+                r = f.getvalue()
+                self.assertEqual(eval(r), a)
+                s = r.strip("bitary(')\n")
+                for group in s.split()[:-1]:
+                    self.assertEqual(len(group), n)
+                for line in s.split('\n'):
+                    self.assertTrue(len(line) < width)
+
     def test_fallback(self):
         for a in None, 'asd', [1, 2], bitarray(), frozenbitarray('1'):
             self.round_trip(a)
@@ -147,7 +168,7 @@ class TestsPPrint(unittest.TestCase):
         self.assertEqual(type(b), type(a))
 
     def test_random(self):
-        for n in range(300):
+        for n in range(150):
             self.round_trip(urandom(n))
 
     def test_file(self):
@@ -163,7 +184,6 @@ class TestsPPrint(unittest.TestCase):
         finally:
             shutil.rmtree(tmpdir)
 
-
 tests.append(TestsPPrint)
 
 # ---------------------------------------------------------------------------
@@ -177,6 +197,7 @@ class TestsMakeEndian(unittest.TestCase, Util):
         c = make_endian(a, 'little')
         self.assertTrue(c == a)
         self.assertEqual(c.endian(), 'little')
+        self.assertIsType(c, 'bitarray')
 
         # wrong arguments
         self.assertRaises(TypeError, make_endian, '', 'big')
@@ -197,6 +218,7 @@ class TestsMakeEndian(unittest.TestCase, Util):
         c = make_endian(a, 'little')
         self.assertTrue(c == a)
         self.assertEqual(c.endian(), 'little')
+        #self.assertIsType(c, 'frozenbitarray')
 
     def test_random(self):
         for a in self.randombitarrays():
@@ -213,7 +235,7 @@ tests.append(TestsMakeEndian)
 
 # ---------------------------------------------------------------------------
 
-class TestsRindex(unittest.TestCase, Util):
+class TestsRIndex(unittest.TestCase, Util):
 
     def test_simple(self):
         self.assertRaises(TypeError, rindex)
@@ -223,18 +245,17 @@ class TestsRindex(unittest.TestCase, Util):
             a = bitarray('00010110000', endian)
             self.assertEqual(rindex(a), 6)
             self.assertEqual(rindex(a, 1), 6)
-            self.assertEqual(rindex(a, 'A'), 6)
-            self.assertEqual(rindex(a, True), 6)
+            self.assertRaises(TypeError, rindex, a, 'A')
+            self.assertRaises(ValueError, rindex, a, 2)
 
             a = bitarray('00010110111', endian)
             self.assertEqual(rindex(a, 0), 7)
-            self.assertEqual(rindex(a, None), 7)
             self.assertEqual(rindex(a, False), 7)
 
             a = frozenbitarray('00010110111', endian)
             self.assertEqual(rindex(a, 0), 7)
-            self.assertEqual(rindex(a, None), 7)
-            self.assertEqual(rindex(a, False), 7)
+            self.assertRaises(TypeError, rindex, a, None)
+            self.assertRaises(ValueError, rindex, a, 7)
 
             for v in 0, 1:
                 self.assertRaises(ValueError, rindex,
@@ -258,26 +279,26 @@ class TestsRindex(unittest.TestCase, Util):
                 j = None
             self.assertEqual(i, j)
 
-    def test_3(self):
-        for _ in range(100):
+    def test_many_set(self):
+        for _ in range(10):
             n = randint(1, 100000)
             v = randint(0, 1)
             a = bitarray(n)
-            a.setall(1 - v)
+            a.setall(not v)
             lst = [randint(0, n - 1) for _ in range(100)]
             for i in lst:
                 a[i] = v
             self.assertEqual(rindex(a, v), max(lst))
 
     def test_one_set(self):
-        for _ in range(100):
+        for _ in range(10):
             N = randint(1, 10000)
             a = bitarray(N)
             a.setall(0)
             a[randint(0, N - 1)] = 1
             self.assertEqual(rindex(a), a.index(1))
 
-tests.append(TestsRindex)
+tests.append(TestsRIndex)
 
 # ---------------------------------------------------------------------------
 
@@ -294,28 +315,52 @@ class TestsStrip(unittest.TestCase, Util):
             self.assertEQUAL(strip(a, 'left'), bitarray('10110000'))
             self.assertEQUAL(strip(a, 'both'), bitarray('1011'))
             b = frozenbitarray('00010110000')
-            self.assertEqual(strip(b, 'both'), bitarray('1011'))
+            c = strip(b, 'both')
+            self.assertEqual(c, bitarray('1011'))
+            self.assertIsType(c, 'frozenbitarray')
 
-        for mode in 'left', 'right', 'both':
-            self.assertEqual(strip(bitarray('000'), mode), bitarray())
-            self.assertEqual(strip(bitarray(), mode), bitarray())
+    def test_zeros(self):
+        for n in range(10):
+            for mode in 'left', 'right', 'both':
+                a = zeros(n)
+                c = strip(a, mode)
+                self.assertIsType(c, 'bitarray')
+                self.assertEqual(c, bitarray())
+                self.assertEqual(a, zeros(n))
+
+                b = frozenbitarray(a)
+                c = strip(b, mode)
+                self.assertIsType(c, 'frozenbitarray')
+                self.assertEqual(c, bitarray())
 
     def test_random(self):
         for a in self.randombitarrays():
             b = a.copy()
+            f = frozenbitarray(a)
             s = a.to01()
-            self.assertEqual(strip(a, 'left'), bitarray(s.lstrip('0')))
-            self.assertEqual(strip(a, 'right'), bitarray(s.rstrip('0')))
-            self.assertEqual(strip(a, 'both'), bitarray(s.strip('0')))
-            self.assertEQUAL(a, b)
+            for mode, res in [
+                    ('left',  bitarray(s.lstrip('0'), a.endian())),
+                    ('right', bitarray(s.rstrip('0'), a.endian())),
+                    ('both',  bitarray(s.strip('0'),  a.endian())),
+            ]:
+                c = strip(a, mode)
+                self.assertEQUAL(c, res)
+                self.assertIsType(c, 'bitarray')
+                self.assertEQUAL(a, b)
+
+                c = strip(f, mode)
+                self.assertEQUAL(c, res)
+                self.assertIsType(c, 'frozenbitarray')
+                self.assertEQUAL(f, b)
 
     def test_one_set(self):
-        for _ in range(100):
-            N = randint(1, 10000)
-            a = bitarray(N)
+        for _ in range(10):
+            n = randint(1, 10000)
+            a = bitarray(n)
             a.setall(0)
-            a[randint(0, N - 1)] = 1
+            a[randint(0, n - 1)] = 1
             self.assertEqual(strip(a, 'both'), bitarray('1'))
+            self.assertEqual(len(a), n)
 
 tests.append(TestsStrip)
 
@@ -529,10 +574,10 @@ class TestsParity(unittest.TestCase, Util):
 
     def test_bitarray(self):
         a = bitarray()
-        self.assertTrue(parity(a) is False)
+        self.assertBitEqual(parity(a), 0)
         par = False
         for _ in range(100):
-            self.assertTrue(parity(a) is par)
+            self.assertEqual(parity(a), par)
             a.append(1)
             par = not par
 
@@ -541,14 +586,10 @@ class TestsParity(unittest.TestCase, Util):
         a.setall(1)
         self.assertTrue(parity(a))
 
-    def test_return_type(self):
-        p = parity(bitarray('101'))
-        self.assertIsInstance(p, bool)
-
     def test_frozenbitarray(self):
-        self.assertTrue(parity(frozenbitarray()) is False)
-        self.assertTrue(parity(frozenbitarray('0010011')) is True)
-        self.assertTrue(parity(frozenbitarray('10100110')) is False)
+        self.assertBitEqual(parity(frozenbitarray()), 0)
+        self.assertBitEqual(parity(frozenbitarray('0010011')), 1)
+        self.assertBitEqual(parity(frozenbitarray('10100110')), 0)
 
     def test_wrong_args(self):
         self.assertRaises(TypeError, parity, '')
@@ -568,20 +609,6 @@ tests.append(TestsParity)
 
 # ---------------------------------------------------------------------------
 
-CODEDICT = {'little': {}, 'big': {
-    '0': bitarray('0000'),    '1': bitarray('0001'),
-    '2': bitarray('0010'),    '3': bitarray('0011'),
-    '4': bitarray('0100'),    '5': bitarray('0101'),
-    '6': bitarray('0110'),    '7': bitarray('0111'),
-    '8': bitarray('1000'),    '9': bitarray('1001'),
-    'a': bitarray('1010'),    'b': bitarray('1011'),
-    'c': bitarray('1100'),    'd': bitarray('1101'),
-    'e': bitarray('1110'),    'f': bitarray('1111'),
-}}
-for k, v in CODEDICT['big'].items():
-    CODEDICT['little'][k] = v[::-1]
-
-
 class TestsHexlify(unittest.TestCase, Util):
 
     def test_ba2hex(self):
@@ -634,16 +661,6 @@ class TestsHexlify(unittest.TestCase, Util):
             for b in b'\0', b'\0f', b'f\0', b'\0ff', b'f\0f', b'ff\0':
                 self.assertRaises(ValueError, hex2ba, b)
 
-    @staticmethod
-    def hex2ba(s, endian=None):
-        a = bitarray(0, endian or get_default_endian())
-        a.encode(CODEDICT[a.endian()], s)
-        return a
-
-    @staticmethod
-    def ba2hex(a):
-        return ''.join(a.iterdecode(CODEDICT[a.endian()]))
-
     def test_explicit(self):
         data = [ #                       little   big
             ('',                         '',      ''),
@@ -660,33 +677,183 @@ class TestsHexlify(unittest.TestCase, Util):
             self.assertEQUAL(hex2ba(hex_le, 'little'), a_le)
             self.assertEqual(ba2hex(a_be), hex_be)
             self.assertEqual(ba2hex(a_le), hex_le)
-            # test simple encode / decode implementation
-            self.assertEQUAL(self.hex2ba(hex_be, 'big'), a_be)
-            self.assertEQUAL(self.hex2ba(hex_le, 'little'), a_le)
-            self.assertEqual(self.ba2hex(a_be), hex_be)
-            self.assertEqual(self.ba2hex(a_le), hex_le)
 
     def test_round_trip(self):
-        for i in range(100):
-            s = ''.join(choice(hexdigits) for _ in range(randint(0, 1000)))
-            for default_endian in 'big', 'little':
-                _set_default_endian(default_endian)
-                a = hex2ba(s)
-                self.assertEqual(len(a) % 4, 0)
-                self.assertEqual(a.endian(), default_endian)
-                t = ba2hex(a)
-                self.assertEqual(t, s.lower())
-                b = hex2ba(t, default_endian)
-                self.assertEQUAL(a, b)
-                # test simple encode / decode implementation
-                self.assertEQUAL(a, self.hex2ba(t))
-                self.assertEqual(t, self.ba2hex(a))
+        s = ''.join(choice(hexdigits) for _ in range(randint(20, 100)))
+        for default_endian in 'big', 'little':
+            _set_default_endian(default_endian)
+            a = hex2ba(s)
+            self.check_obj(a)
+            self.assertEqual(len(a) % 4, 0)
+            self.assertEqual(a.endian(), default_endian)
+            t = ba2hex(a)
+            self.assertEqual(t, s.lower())
+            b = hex2ba(t, default_endian)
+            self.assertEQUAL(a, b)
 
+    def test_binascii(self):
+        a = urandom(800, 'big')
+        s = binascii.hexlify(a.tobytes()).decode()
+        self.assertEqual(ba2hex(a), s)
+        b = bitarray(endian='big')
+        b.frombytes(binascii.unhexlify(s))
+        self.assertEQUAL(hex2ba(s, 'big'), b)
 
 tests.append(TestsHexlify)
 
 # ---------------------------------------------------------------------------
 
+class TestsBase(unittest.TestCase, Util):
+
+    def test_ba2base(self):
+        c = ba2base(16, bitarray('1101', 'big'))
+        self.assertIsInstance(c, str)
+
+    def test_base2ba(self):
+        _set_default_endian('big')
+        for c in 'e', 'E', b'e', b'E', u'e', u'E':
+            a = base2ba(16, c)
+            self.assertEqual(a.to01(), '1110')
+            self.assertEqual(a.endian(), 'big')
+
+    def test_explicit(self):
+        data = [ #              n  little   big
+            ('',                2, '',      ''),
+            ('1 0 1',           2, '101',   '101'),
+            ('11 01 00',        4, '320',   '310'),
+            ('111 001',         8, '74',    '71'),
+            ('1111 0001',      16, 'f8',    'f1'),
+            ('11111 00001',    32, '7Q',    '7B'),
+            ('111111 000001',  64, '/g',    '/B'),
+        ]
+        for bs, n, s_le, s_be in data:
+            a_le = bitarray(bs, 'little')
+            a_be = bitarray(bs, 'big')
+            self.assertEQUAL(base2ba(n, s_le, 'little'), a_le)
+            self.assertEQUAL(base2ba(n, s_be, 'big'),    a_be)
+            self.assertEqual(ba2base(n, a_le), s_le)
+            self.assertEqual(ba2base(n, a_be), s_be)
+
+    def test_empty(self):
+        for n in 2, 4, 8, 16, 32, 64:
+            a = base2ba(n, '')
+            self.assertEqual(a, bitarray())
+            self.assertEqual(ba2base(n, a), '')
+
+    def test_upper(self):
+        self.assertEqual(base2ba(16, 'F'), bitarray('1111'))
+
+    def test_invalid_characters(self):
+        for n, s in ((2, '2'), (4, '4'), (8, '8'), (16, 'g'), (32, '8'),
+                     (32, '1'), (32, 'a'), (64, '-'), (64, '_')):
+            self.assertRaises(ValueError, base2ba, n, s)
+
+    def test_invalid_args(self):
+        a = bitarray()
+        self.assertRaises(TypeError, ba2base, None, a)
+        self.assertRaises(TypeError, base2ba, None, '')
+        for i in range(-10, 260):
+            if i in (2, 4, 8, 16, 32, 64):
+                continue
+            self.assertRaises(ValueError, ba2base, i, a)
+            self.assertRaises(ValueError, base2ba, i, '')
+
+    def test_binary(self):
+        a = base2ba(2, '1011')
+        self.assertEqual(a, bitarray('1011'))
+        self.assertEqual(ba2base(2, a), '1011')
+
+        for a in self.randombitarrays():
+            s = ba2base(2, a)
+            self.assertEqual(s, a.to01())
+            self.assertEQUAL(base2ba(2, s, a.endian()), a)
+
+    def test_quaternary(self):
+        a = base2ba(4, '0123', 'big')
+        self.assertEqual(a, bitarray('00 01 10 11'))
+        self.assertEqual(ba2base(4, a), '0123')
+
+    def test_octal(self):
+        a = base2ba(8, '0147', 'big')
+        self.assertEqual(a, bitarray('000 001 100 111'))
+        self.assertEqual(ba2base(8, a), '0147')
+
+    def test_hexadecimal(self):
+        a = base2ba(16, 'F61', 'big')
+        self.assertEqual(a, bitarray('1111 0110 0001'))
+        self.assertEqual(ba2base(16, a), 'f61')
+
+        for n in range(50):
+            s = ''.join(choice(hexdigits) for _ in range(n))
+            for endian in 'big', 'little':
+                a = base2ba(16, s, endian)
+                self.assertEQUAL(a, hex2ba(s, endian))
+                self.assertEqual(ba2base(16, a), ba2hex(a))
+
+    def test_base32(self):
+        a = base2ba(32, '7SH', 'big')
+        self.assertEqual(a, bitarray('11111 10010 00111'))
+        self.assertEqual(ba2base(32, a), '7SH')
+
+        msg = os.urandom(randint(10, 100) * 5)
+        s = base64.b32encode(msg).decode()
+        a = base2ba(32, s, 'big')
+        self.assertEqual(a.tobytes(), msg)
+        self.assertEqual(ba2base(32, a), s)
+
+    def test_base64(self):
+        a = base2ba(64, '/jH', 'big')
+        self.assertEqual(a, bitarray('111111 100011 000111'))
+        self.assertEqual(ba2base(64, a), '/jH')
+
+        msg = os.urandom(randint(10, 100) * 3)
+        s = base64.standard_b64encode(msg).decode()
+        a = base2ba(64, s, 'big')
+        self.assertEqual(a.tobytes(), msg)
+        self.assertEqual(ba2base(64, a), s)
+
+    def test_alphabets(self):
+        for m, n, alpabet in [
+                (1,  2, '01'),
+                (2,  4, '0123'),
+                (3,  8, '01234567'),
+                (4, 16, '0123456789abcdef'),
+                (5, 32, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ234567'),
+                (6, 64, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
+                        'abcdefghijklmnopqrstuvwxyz0123456789+/'),
+        ]:
+            self.assertEqual(1 << m, n)
+            self.assertEqual(len(alpabet), n)
+            for i, c in enumerate(alpabet):
+                for endian in 'big', 'little':
+                    self.assertEqual(ba2int(base2ba(n, c, endian)), i)
+                    self.assertEqual(ba2base(n, int2ba(i, m, endian)), c)
+
+    def test_random(self):
+        for a in self.randombitarrays():
+            for m in range(1, 7):
+                n = 1 << m
+                if len(a) % m == 0:
+                    s = ba2base(n, a)
+                    b = base2ba(n, s, a.endian())
+                    self.assertEQUAL(a, b)
+                    self.check_obj(b)
+                else:
+                    self.assertRaises(ValueError, ba2base, n, a)
+
+    def test_random2(self):
+        for m in range(1, 7):
+            n = 1 << m
+            for length in range(0, 100, m):
+                a = urandom(length, 'little')
+                self.assertEQUAL(base2ba(n, ba2base(n, a), 'little'), a)
+                b = bitarray(a, 'big')
+                self.assertEQUAL(base2ba(n, ba2base(n, b), 'big'), b)
+
+tests.append(TestsBase)
+
+# ---------------------------------------------------------------------------
+
 class TestsIntegerization(unittest.TestCase, Util):
 
     def test_ba2int(self):
@@ -819,6 +986,7 @@ class TestsIntegerization(unittest.TestCase, Util):
     def check_round_trip(self, i):
         for endian in 'big', 'little':
             a = int2ba(i, endian=endian)
+            self.check_obj(a)
             self.assertEqual(a.endian(), endian)
             self.assertTrue(len(a) > 0)
             # ensure we have no leading zeros
@@ -836,7 +1004,7 @@ class TestsIntegerization(unittest.TestCase, Util):
             self.assertEqual(ba2int(a), i)
 
     def test_many(self):
-        for i in range(100):
+        for i in range(20):
             self.check_round_trip(i)
             self.check_round_trip(randint(0, 10 ** randint(3, 300)))
 
@@ -858,11 +1026,94 @@ class TestsIntegerization(unittest.TestCase, Util):
 
             self.assertEqual(i, self.twos_complement(j, len(a)))
 
-
 tests.append(TestsIntegerization)
 
 # ---------------------------------------------------------------------------
 
+class MixedTests(unittest.TestCase, Util):
+
+    def test_bin(self):
+        for i in range(100):
+            s = bin(i)
+            self.assertEqual(s[:2], '0b')
+            a = bitarray(s[2:], 'big')
+            self.assertEqual(ba2int(a), i)
+            t = '0b%s' % a.to01()
+            self.assertEqual(t, s)
+            self.assertEqual(eval(t), i)
+
+    def test_oct(self):
+        if sys.version_info[0] == 2:
+            return
+        for i in range(1000):
+            s = oct(i)
+            self.assertEqual(s[:2], '0o')
+            a = base2ba(8, s[2:], 'big')
+            self.assertEqual(ba2int(a), i)
+            t = '0o%s' % ba2base(8, a)
+            self.assertEqual(t, s)
+            self.assertEqual(eval(t), i)
+
+    def test_hex(self):
+        for i in range(1000):
+            s = hex(i)
+            self.assertEqual(s[:2], '0x')
+            a = hex2ba(s[2:], 'big')
+            self.assertEqual(ba2int(a), i)
+            t = '0x%s' % ba2hex(a)
+            self.assertEqual(t, s)
+            self.assertEqual(eval(t), i)
+
+    def test_bitwise(self):
+        for a in self.randombitarrays(start=1):
+            b = urandom(len(a), a.endian())
+            aa = a.copy()
+            bb = b.copy()
+            i = ba2int(a)
+            j = ba2int(b)
+            self.assertEqual(ba2int(a & b), i & j)
+            self.assertEqual(ba2int(a | b), i | j)
+            self.assertEqual(ba2int(a ^ b), i ^ j)
+
+            n = randint(0, len(a))
+            if a.endian() == 'big':
+                self.assertEqual(ba2int(a >> n), i >> n)
+                c = zeros(len(a)) + a
+                self.assertEqual(ba2int(c << n), i << n)
+
+            self.assertEQUAL(a, aa)
+            self.assertEQUAL(b, bb)
+
+    def test_bitwise_inplace(self):
+        for a in self.randombitarrays(start=1):
+            b = urandom(len(a), a.endian())
+            bb = b.copy()
+            i = ba2int(a)
+            j = ba2int(b)
+            c = a.copy()
+            c &= b
+            self.assertEqual(ba2int(c), i & j)
+            c = a.copy()
+            c |= b
+            self.assertEqual(ba2int(c), i | j)
+            c = a.copy()
+            c ^= b
+            self.assertEqual(ba2int(c), i ^ j)
+            self.assertEQUAL(b, bb)
+
+            n = randint(0, len(a))
+            if a.endian() == 'big':
+                c = a.copy()
+                c >>= n
+                self.assertEqual(ba2int(c), i >> n)
+                c = zeros(len(a)) + a
+                c <<= n
+                self.assertEqual(ba2int(c), i << n)
+
+tests.append(MixedTests)
+
+# ---------------------------------------------------------------------------
+
 class TestsSerialization(unittest.TestCase, Util):
 
     def test_explicit(self):
@@ -925,7 +1176,7 @@ class TestsSerialization(unittest.TestCase, Util):
         for a in self.randombitarrays():
             b = deserialize(serialize(a))
             self.assertEQUAL(a, b)
-
+            self.check_obj(b)
 
 tests.append(TestsSerialization)
 
@@ -1022,10 +1273,9 @@ tests.append(TestsHuffman)
 # ---------------------------------------------------------------------------
 
 def run(verbosity=1):
-    import os
     import bitarray
 
-    print('bitarray is installed in: %s' % os.path.dirname(bitarray.__file__))
+    print('bitarray.util is installed in: %s' % os.path.dirname(__file__))
     print('bitarray version: %s' % bitarray.__version__)
     print('Python version: %s' % sys.version)
 
