@@ -254,8 +254,6 @@ the `length` of the bitarray is provided.  An `OverflowError` is raised
 if the integer is not representable with the given number of bits.
 `signed` determines whether two's complement is used to represent the integer,
 and requires `length` to be provided.
-If signed is False and a negative integer is given, an OverflowError
-is raised.
 """
     if not isinstance(i, (int, long) if _is_py2 else int):
         raise TypeError("int expected, got '%s'" % type(i).__name__)
@@ -272,12 +270,18 @@ is raised.
         return zeros(length or 1, endian)
 
     if signed:
-        if i >= 1 << (length - 1) or i < -(1 << (length - 1)):
-            raise OverflowError("signed integer out of range")
+        m = 1 << (length - 1)
+        if not (-m <= i < m):
+            raise OverflowError("signed integer not in range(%d, %d), "
+                                "got %d" % (-m, m, i))
         if i < 0:
             i += 1 << length
-    elif i < 0 or (length and i >= 1 << length):
-        raise OverflowError("unsigned integer out of range")
+    else:  # unsigned
+        if i < 0:
+            raise OverflowError("unsigned integer not positive, got %d" % i)
+        if length and i >= (1 << length):
+            raise OverflowError("unsigned integer not in range(0, %d), "
+                                "got %d" % (1 << length, i))
 
     a = bitarray(0, get_default_endian() if endian is None else endian)
     big_endian = bool(a.endian() == 'big')
@@ -327,8 +331,8 @@ def huffman_code(freq_map, endian=None):
 
 Given a frequency map, a dictionary mapping symbols to their frequency,
 calculate the Huffman code, i.e. a dict mapping those symbols to
-bitarrays (with given endianness).  Note that the symbols may be any
-hashable object (including `None`).
+bitarrays (with given endianness).  Note that the symbols are not limited
+to being strings.  Symbols may may be any hashable object (such as `None`).
 """
     import heapq
 
